﻿-- Thanks for the fix for Noctodemus!

-- This fix adds functionality to automatically correct the difficulty setting quirks of several 
-- legacy instances. There are	two potential sources of trouble:
-- 	1) An instance may arbitrarily override player difficulty settings. In some cases, you can
-- 		see this before entering the instance because the 'heroic' skull is missing even though
-- 		heroic raid difficulty is selected.
-- 	2) An instance doesn't have a difficulty setting to match the one currently selected. When
-- 		this happens, the instance may behave normally but on exit the player's settings have
-- 		been silently changed.
-- 	This fix addresses both of these conditions and the player's difficulty settings should
-- 	remain as they are whenever possible.
-- 
--  A few known problem areas are:
-- 	- Icecrown Citadel - Sometimes sets difficulty to normal regardless of player settings.
-- 	- Throne of the Four Winds - Same as Icecrown.
-- 	- Any 10-man instance when the player has selected 25-man - Will leave size at 10-man.

local addon = CreateFrame("Frame", "TDFrame")
local iType, iDiff

local TD_DropDown = CreateFrame("Frame", "TDDropDownFrame", addon, "L_UIDropDownMenuTemplate")

local DiffsShort = { "N", "H", "10", "25", "10H", "25H", "LFR", "CM", "40", nil, "HSC", "SC", nil, "N", "H", "M", "LFR", "E", "E", nil, nil, nil, "M"}
local DungeonDiffs = { DUNGEON_DIFFICULTY1, DUNGEON_DIFFICULTY2, nil, nil, nil, nil, nil, CHALLENGE_MODE, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, PLAYER_DIFFICULTY6 }
local RaidDiffs = { nil, nil, RAID_DIFFICULTY1, RAID_DIFFICULTY2, RAID_DIFFICULTY3, RAID_DIFFICULTY4 ,nil ,nil ,nil ,nil ,nil ,nil , nil, PLAYER_DIFFICULTY1, PLAYER_DIFFICULTY2, PLAYER_DIFFICULTY6}

local TDobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Dungeon Difficulty", {
    type = "data source",
    text = "None",
    icon = "Interface\\Icons\\Achievement_Dungeon_TheVioletHold_25man",
    OnClick = function(self, button)
        if button == "RightButton" then
			GameTooltip:Hide()
            L_ToggleDropDownMenu(1, "TDDropDownFrame", TD_DropDown, self, 0, 0)
        else
			ToggleRaidFrame();
        end
    end,
    } )

function addon:new()
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_ALIVE");
	self:RegisterEvent("CHAT_MSG_SYSTEM");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("GUILD_PARTY_STATE_UPDATED");
	self:RegisterEvent("PARTY_MEMBER_DISABLE");
	self:RegisterEvent("PARTY_MEMBER_ENABLE");
	self:RegisterEvent("PLAYER_DIFFICULTY_CHANGED");
	self:RegisterEvent("PLAYER_GUILD_UPDATE");
	self:RegisterEvent("UPDATE_INSTANCE_INFO");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:SetScript("OnEvent", self.onevent)
end

function Get_Diffs()
	dDiff = GetDungeonDifficultyID() or 1
	lDiff = GetLegacyRaidDifficultyID() or 3
	rDiff = GetRaidDifficultyID() or 14
end

function Save_Diffs()
	_, iType, iDiff = GetInstanceInfo()
	
	if iType == "none" then
		dSave = GetDungeonDifficultyID()
		lSave = GetLegacyRaidDifficultyID()
		rSave = GetRaidDifficultyID()
	end
end

function Restore_Diffs()
	
	local settingsChanged = false
	
	_, iType, iDiff, _, _, _, iDynamic = GetInstanceInfo()

	if iType == "none" then
		if GetDungeonDifficultyID ~= dSave then
			SetDungeonDifficultyID(dSave)
			settingsChanged = true
		end
		
		if GetLegacyRaidDifficultyID ~= lSave then
			SetLegacyRaidDifficultyID(lSave)
			settingsChanged = true
		end
		
		if GetRaidDifficultyID ~= rSave then
			SetRaidDifficultyID(rSave)
			settingsChanged = true
		end
		
		if settingsChanged then
			BDTextMessage("Restoring Difficulty Settings")
		end
	elseif iDynamic and (iType == "party" or iType == "raid") and UnitIsGroupLeader("player") then
		if GetDungeonDifficultyID() ~= dSave then
			SetDungeonDifficultyID(dSave)
			settingsChanged = true
		end
		
		if GetLegacyRaidDifficultyID() ~= lSave then
			SetLegacyRaidDifficultyID(lSave)
			settingsChanged = true
		end
		
		if GetRaidDifficultyID() ~= rSave then
			SetRaidDifficultyID(rSave)
			settingsChanged = true
		end
		
		if settingsChanged then
			BDTextMessage("Attempting to Set Instance Difficulty")
		end
	end
end

function UpdateTDobj()
	Get_Diffs()
	if DiffsShort[dDiff] ~= nil and DiffsShort[lDiff] ~= nil and DiffsShort[rDiff] ~= nil then
		TDobj.text = DiffsShort[dDiff] .. "/" .. DiffsShort[rDiff] .. "/" .. DiffsShort[lDiff]
	end
end

function addon:onevent(event, data1)
	if event == "PLAYER_LOGIN" then
		Save_Diffs()
		UpdateTDobj()
		L_UIDropDownMenu_Initialize(TD_DropDown, TD_DropDownInit, "MENU")
	elseif event == "PLAYER_ENTERING_WORLD" then
		Restore_Diffs()
		UpdateTDobj()
	elseif event == "CHAT_MSG_SYSTEM" then
		if string.find(data1, DUNGEON_DIFFICULTY) or string.find(data1, RAID_DIFFICULTY) then
			UpdateTDobj()
			C_Timer.After(1, Save_Diffs)
		end
	else
		UpdateTDobj()
		C_Timer.After(1, Save_Diffs)
	end
end


------------------------
--      Tooltip!      --
------------------------

function TDobj.OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
    GameTooltip:ClearLines()
    TDobj.OnTooltipShow(GameTooltip)
    GameTooltip:Show()
end

function TDobj.OnTooltipShow(self)
    self:AddLine("Current Difficulties")
    self:AddDoubleLine("Dungeons:", GetDifficultyInfo(dDiff), 1,1,1, 1,1,1);
    self:AddDoubleLine("Raids:", GetDifficultyInfo(rDiff), 1,1,1, 1,1,1);
    self:AddDoubleLine("Legacy Raids:", GetDifficultyInfo(lDiff), 1,1,1, 1,1,1);
    self:AddLine("")
    self:AddLine("|cff00ff00" .. "Left-click to toggle raid frame." .. "|r")
    self:AddLine("|cff00ff00" .. "Right-click to change difficulty." .. "|r")
end


function TDobj.OnLeave()
	GameTooltip:Hide()
end

function ToggleShowMessages()
	iShowMessages = not iShowMessages
end


------------------------
--        Menu!       --
------------------------

function TD_DropDownInit(self, level, menu)
    level = level or 1
    
    local info = L_UIDropDownMenu_CreateInfo()
    local index = 1
    local inInstanceGroup = IsInGroup(LE_PARTY_CATEGORY_INSTANCE)
    local inGroup = IsInGroup()
    
    if level == 1 then
		wipe(info)
        info.text = "Settings"
        info.notCheckable = true
        info.isTitle = true
        L_UIDropDownMenu_AddButton(info, level)
		
		wipe(info)
		info.text = "Show Messages"
		info.isNotRadio = true
		info.checked = (iShowMessages == true)
		info.func = ToggleShowMessages
		L_UIDropDownMenu_AddButton(info, level)

		wipe(info)
        info.text = DUNGEON_DIFFICULTY
        info.notCheckable = true
        info.isTitle = true
        L_UIDropDownMenu_AddButton(info, level)

		for i=1, 23 do
			if DungeonDiffs[i] ~= nil then
				wipe(info)
				info.text = DungeonDiffs[i]
				info.value = i
				info.checked = (dDiff == i) 
				info.func = function(self, diff) SetDungeonDifficultyID(i) end
				L_UIDropDownMenu_AddButton(info, level)
			end
		end

        wipe(info)
        info.text = RAID_DIFFICULTY
        info.notCheckable = true
        info.isTitle = true
        L_UIDropDownMenu_AddButton(info, level)
  		for i=14, 16 do
			wipe(info)
			info.text = RaidDiffs[i]
			info.value = i
			info.checked = (rDiff == i)
			info.func = function(self, diff) SetRaidDifficultyID(i) end
			L_UIDropDownMenu_AddButton(info, level)
		end
       
        wipe(info)
        info.text = LEGACY_RAID_DIFFICULTY
        info.notCheckable = true
        info.isTitle = true
        L_UIDropDownMenu_AddButton(info, level)
		for i=3, 6 do
			wipe(info)
			info.text = RaidDiffs[i]
			info.value = i
			info.checked = (lDiff == i)
			info.func = function(self, diff) SetLegacyRaidDifficultyID(i) end
			L_UIDropDownMenu_AddButton(info, level)
		end
		if inInstanceGroup or inGroup then
			wipe(info)
			info.text = OTHER
			info.notCheckable = true
			info.isTitle = true
			L_UIDropDownMenu_AddButton(info, level)
			if inInstanceGroup then
				wipe(info)
				info.text = INSTANCE_PARTY_LEAVE
				info.notCheckable = true
				info.func = function() ConfirmOrLeaveLFGParty() end
				L_UIDropDownMenu_AddButton(info, level)
			elseif inGroup then
				wipe(info)
				info.text = PARTY_LEAVE
				info.notCheckable = true
				info.func = function() LeaveParty() end
				L_UIDropDownMenu_AddButton(info, level)
			end
		end
    end
end

------------------------
--    Fading Text!    --
------------------------

function BDTextMessage(message)
	if iShowMessages then
		UIErrorsFrame:AddMessage("Dungeon Difficulty - " .. message, 1.0, 1.0, 0.0);
		print("Dungeon Difficulty - " .. message)
	end
end

addon:new()